#!/usr/bin/env python
##
## Awesum - Graphical Checksum Verifier
##
## Copyright (c) 2004, Darren L. LaChausse
## ALL RIGHTS RESERVED
##
## Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions
## are met:
## 1. Redistributions of source code must retain the above copyright
##    notice, this list of conditions and the following disclaimer.
## 2. Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
## 3. Neither the name of the Author nor the names of contributors
##    may be used to endorse or promote products derived from this software
##    without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
## ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
## IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
## ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
## FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
## DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
## OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
## HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
## LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
## OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
## SUCH DAMAGE.
##

######################################################################
######################################################################
##
## CONFIGURATION DATA
##
######################################################################
######################################################################

#Version information
version_major 		= "0"
version_minor 		= "6"
version_revision 	= "0"
version_codename	= "Golden God"
version_projectname	= "Awesum"

#Now put it all together and repeat after me...
version_string = 	version_projectname + " " + \
			version_major + "." + \
			version_minor + "." + \
			version_revision + " [" + \
			version_codename + "]"

#Copyright information
author_string		= "Darren L. LaChausse"
copyright_year		= "2004"
copyright_string	= "Copyright (c) " + copyright_year + ", " + \
				author_string

######################################################################
######################################################################
##
## GENERAL INITIALIZATION
##
######################################################################
######################################################################

import sys, md5, sha, getopt

try:
	import pygtk
	#Try to get GTK 2 if we can...
	pygtk.require("2.0")
except:
	#...but it's okay if we can't
	pass

try:
	import gtk
	import gtk.glade
except:
	#Tell the user what we need and then bail
	print "This program requires PyGTK or GTKv2.  Sorry."
	sys.exit(1)

######################################################################
######################################################################
##
## UTILITY FUNCTIONS
##
######################################################################
######################################################################

def do_checksum(filename, csum, algorithm):
	#Open the file
	f = file(filename, "r")

	#Find out the size of the file
	f.seek(0, 2)
	size = f.tell()
	f.seek(0, 0)

	#Create the appropriate checksum object
	if(algorithm == "MD5"):
		cs = md5.new()
	elif(algorithm == "SHA"):
		cs = sha.new()
	else:
		raise "Invalid Checksum Algorithm Specified"

	#Initialize the progress dialog box
	init_progress()

	#Set a sensible blocksize
	bs = size / 100

	block = f.read(bs)
	while(block != ""):
		cs.update(block)
		update_progress((f.tell() * 1.0) / size)
		block = f.read(bs)
	close_progress()

	#See if we got the right answer
	if(cs.hexdigest() == csum):
		return 1
	else:
		return 0

def init_progress():
	global progwin, progbar
	progwin = gtk.glade.XML("awesum.glade", "progwin")
	progbar = progwin.get_widget("progress")
	return

def close_progress():
	progwin.get_widget("progwin").destroy()
	return

def update_progress(status):
	#Update the progress bar
	progbar.set_fraction(status)

	#Make sure the window gets a chance to redraw
	while gtk.events_pending():
		gtk.mainiteration();
	return

def get_filename():
	fileselect = gtk.glade.XML("awesum.glade", "fileselect")
	fileselect.signal_autoconnect({
		"on_fs_cancel_button_clicked" : fs_cancel_button_clicked
		})

	global fileselectdlg	#We need this to be global for the cancel button
	fileselectdlg = fileselect.get_widget("fileselect")

	result = fileselectdlg.run()

	#If the user selected a file, return its name to the caller
	if(result != gtk.RESPONSE_NONE and result != gtk.RESPONSE_DELETE_EVENT):
		ret = fileselect.get_widget("fileselect").get_filename()
	else:
	#Otherwise return null
		ret = ""
	
	#Hide the dialog from the user since we no longer need it
	fileselectdlg.hide_all()
	return ret

######################################################################
######################################################################
##
## GTK CALLBACKS
##
######################################################################
######################################################################

def mainwin_delete_event(self, widget):
	button_quit_clicked(widget)
	return

def button_quit_clicked(widget):
	gtk.mainquit()
	return

def button_go_clicked(widget):
	#Get the expected checksum entered by the user
	checksum = gladeXML.get_widget("expected_csum").get_text()
	if(checksum == ""):	
		#Tell the user to enter a checksum value
		msgdlg = gtk.MessageDialog(
			None,
			gtk.DIALOG_MODAL,
			gtk.MESSAGE_ERROR,
			gtk.BUTTONS_CLOSE,
			"Please specify a checksum value."
			)
		msgdlg.run()
		msgdlg.destroy()

		return

	#Find out what algorithm to use
	history = gladeXML.get_widget("csum_type").get_history()
	if(history == 0):
		algorithm = "MD5"
	elif(history == 1):
		algorithm = "SHA"
	else:
		#This should never happen, so we'll bail
		raise "Oops! csum_type out of range"

	#Get the filename from the user
	filename = get_filename()

	if(filename != ""):
		if(do_checksum(filename, checksum, algorithm)):
			#The checksum is good
			msgdlg = gtk.MessageDialog(
				None,
				gtk.DIALOG_MODAL,
				gtk.MESSAGE_INFO,
				gtk.BUTTONS_OK,
				"Checksum verification successful!"
				)
			msgdlg.run()
			msgdlg.destroy()
		else:
			#No good, tell the user to try again
			msgdlg = gtk.MessageDialog(
				None,
				gtk.DIALOG_MODAL,
				gtk.MESSAGE_ERROR,
				gtk.BUTTONS_CLOSE,
				"Checksum values do not match."
				)
			msgdlg.run()
			msgdlg.destroy()

	return

def fs_cancel_button_clicked(widget):
	fileselectdlg.response(gtk.RESPONSE_NONE)
	return

######################################################################
######################################################################
##
## GTK INITIALIZATION
##
######################################################################
######################################################################

def initialize_gtk():
	#Load the main window from the glade file
	global gladeXML
	gladeXML = gtk.glade.XML("awesum.glade", "mainwin")

	#Initialize callbacks
	gladeXML.signal_autoconnect({
		"on_button_quit_clicked"	: button_quit_clicked,
		"on_mainwin_delete_event"	: mainwin_delete_event,
		"on_button_go_clicked"		: button_go_clicked
		})

	#Okay PyGTK, do your stuff
	gtk.mainloop()

######################################################################
######################################################################
##
## USAGE INFORMATION
##
######################################################################
######################################################################

def print_usage():
	print """
Usage: awesum [OPTIONS]

Where OPTIONS can be one of the following:

   -a, --about          print a short about string (for inclusion in
                           shell scripts)
   -v, --version        print full version and copyright information
   -h, --help, --usage  display this help message and exit
"""

######################################################################
######################################################################
##
## MAIN
##
######################################################################
######################################################################

#Parse command line options
try:
	opts, args = getopt.getopt(sys.argv[1:], "avh", 
					["about", "version", "help", "usage"])
except getopt.GetoptError:
	#Print usage information and exit
	print_usage()
	sys.exit(1)

for o, a in opts:
	if o in ("-a", "--about"):
		#Short about string (for use by shell scripts)
		print	version_major + "." + \
			version_minor + "." + \
			version_revision
		sys.exit(1)

	if o in ("-v", "--version"):
		#Display full version information
		print "\n" + version_string
		print copyright_string + "\n"
		sys.exit(1)

	if o in ("-h", "--help", "--usage"):
		#Print usage information and exit
		print_usage()
		sys.exit(1)
		
#If we get this far, let 'er rip
initialize_gtk()
